# Arduino-библиотека для датчика MS5540C

## Краткая справка по датчику

Данный датчик использует модифицированный протокол SPI.
Главное отличие протокола в том, что датчик не имеет
шины SS/CS (Slave Select/Chip Select).
Вместо этого датчик принимает 3 стартовых и 3 стоповых бита
до и после каждой управляющей последовательности.

Датчик способен работать на заявленной глубине до 100м.

## Пример кода

Пример иллюстрирует работу датчика ms5540c.
Скетч считывает текущие температуру и давление каждые три секунды.

*Также, если вы не знаете микроконтроллеры AVR, как свои пять
пальцев, то настоятельно рекомендуется прочесть раздел
"Важные замечания" во избежание различных сложных для понимания
проблем.

```ino
#include <SPI.h>
#include <ms5540c.h>

ms5540c sensor;

void setup() {
    Serial.begin(9600);
    SPI.begin();
    sensor.init();
    Serial.println("Temp\t\tmbar\t\tatm\t\tpascal\n");
}

void loop() {
    const long temp_raw = sensor.getTemperature(); // Temperature in tenths of the deg C
    const long prs_raw = sensor.getPressure(); // pressure in tenths of a mbar (because of the sensor precision)
    Serial.print(conv::degC(temp_raw));                 Serial.print("\t\t");
    Serial.print(conv::mbar(prs_raw));                  Serial.print("\t\t");
    Serial.print(conv::mbarToAtm(conv::mbar(prs_raw))); Serial.print("\t\t");
    Serial.println(conv::mbarToPascal(conv::mbar(prs_raw)));

    delay(3000);
}
```

### Пояснения к коду

```ino
SPI.begin(<desired_baud_rate>);
```

Библиотека не запускает коммуникацию через интерфейс SPI,
поэтому необходимо запустить её самостоятельно.

```ino
const long temp_raw = sensor.getTemperature(); // Temperature in tenths of the deg C
const long prs_raw = sensor.getPressure(); // pressure in tenths of a mbar (because of the sensor precision)
```

Так как заявленная точность датчика составляет 1/10 миллибар
и 1/10 градуса Цельсия для давления и температуры соответственно,
то показания возвращаются с датчика в количестве десятых долей
миллибар и градусов Цельсия.

## Важные замечания

### Что данная библиотека делает и не делает

Библиотека принимает и передаёт данные пользователю "как есть",
то есть не выполняет никаких сложных преобразований над
полученными данными (кроме двухуровневой температурной
компенсации измерений).
Также, библиотека не поддерживает арифметику чисел с плавающей
точкой, потому что микроконтроллеры AVR (по крайней мере, те,
которые используются в платах Arduino) не поддерживают таковую
на аппаратном уровне.
Таким образом, конверсия измеренных значений внутри библиотеки
к формату с плавающей точкой создаёт странные и неожиданные
проблемы при попытке каким-либо образом манипулировать
полученными данными (преобразовывать, записывать
на внешний носитель и т.д.).
Тем не менее, библиотека предоставляет функции конверсии
между единицами измерения давления, расположенные в пространстве
имен `conv` (см. раздел ниже).

Крайне рекомендуется перенести вычисления и обработку чисел
с плавающей точкой в какие-либо скрипты, программы и т.п.,
исполняемые на ПК.

### Особенности подключения к микроконтроллеру

Прежде всего, библиотека использует аппаратную реалицазию протокола SPI, так что необходимо
соотнестись со схемой разводки конкретно вашего микроконтроллера, чтобы убедиться в том, что
он поддерживает протокол и имеет выводы MISO/MOSI/SCK.

**Крайне важное замечание:** датчик не имеет собственного
генератора частоты, поэтому ему необходимо предоставить
внешний источник тактовой частоты.
Таковой подключается через вывод `MCLK`.
Убедитесь, что вы подключаете его к разъему, на который
выведен выход `OC1A` микроконтроллера (обращайтесь к схеме
разводки вашей платы Arduino).

*Подсказка: если вы используете Arduino Uno R3, то таковой
выход выведен на пин D9.

Подключение этого разъема к какому-либо другому пину
приведет к неправильной работе датчика, и вы будете получать
от него неверные значения измерений (вроде отрицательного
давления).

Замечание: шины передачи данных (MOSI/MISO) лучше подтянуть
соответствующими резисторами, дабы избавиться от шумов на линии.

Замечание: рекомендуется подключать последовательные шины через подтягивающие резисторы,
чтобы избавиться от шума на линии.

### Функции преобразования

|    Название функции             | Что делает                                   |
|---------------------------------|----------------------------------------------|
| `conv::mbar(long prs_raw)`      | Преобр. "сырые" данные давления в мбар       |
| `conv::degC(long temp_raw)`     | Преобр. "сырые" данные температуры в градусы |
| `conv::mbarTommHg(long mbar)`   | mbar -> mmHg                                 |
| `conv::mbarToAtm(long mbar)`    | mbar -> atm                                  |
| `conv::mbarToPascal(long mbar)` | mbar -> pascal                               |
